<?php

namespace FiloBlu\Refilo\Remote;

use FiloBlu\Refilo\Remote\Entity\EntityProviderInterfaceFactory;
use FiloBlu\Refilo\Remote\Helper\TemplateStringFactory;
use FiloBlu\Refilo\Remote\IndexerConfiguration\IndexerConfigurationRepositoryInterface;
use FiloBlu\Refilo\Remote\IndexerConfiguration\IndexerConfigurationInterfaceFactory;
use FiloBlu\Refilo\Remote\IndexerConfiguration\IndexerResolver;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\DataObjectFactory;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Store\Model\StoreManagerInterface;
use Psr\Log\LoggerInterface;
/**
 * Class PriceIndexer
 * @package FiloBlu\Refilo\Remote
 */
//@todo check for empty bulk if extended from AbstractWebsiteBasedIndexer
class PriceIndexer extends AbstractStoreBasedIndexer
{
    /**
     * @var ResourceConnection
     */
    private $resourceConnection;

    /**
     * PriceIndexer constructor.
     * @param StoreManagerInterface $storeManager
     * @param EntityProviderInterfaceFactory $entityProviderFactory
     * @param IndexerConfigurationRepositoryInterface $indexerConfigurationRepository
     * @param TemplateStringFactory $templateStringFactory
     * @param IndexerResolver $indexerResolver
     * @param ScopeConfigInterface $scopeConfig
     * @param LoggerInterface $logger
     * @param ResourceConnection $resourceConnection
     * @param DataObjectFactory $dataObjectFactory
     * @param IndexerConfigurationInterfaceFactory $indexerConfigurationFactory
     */
    public function __construct(
        StoreManagerInterface $storeManager,
        EntityProviderInterfaceFactory $entityProviderFactory,
        IndexerConfigurationRepositoryInterface $indexerConfigurationRepository,
        TemplateStringFactory $templateStringFactory,
        IndexerResolver $indexerResolver,
        ScopeConfigInterface $scopeConfig,
        LoggerInterface $logger,
        ResourceConnection $resourceConnection,
        DataObjectFactory $dataObjectFactory,
        IndexerConfigurationInterfaceFactory $indexerConfigurationFactory
    ) {
        parent::__construct($storeManager,
            $entityProviderFactory,
            $indexerConfigurationRepository,
            $templateStringFactory,
            $indexerResolver,
            $scopeConfig,
            $logger,
            $dataObjectFactory,
            $indexerConfigurationFactory);
        $this->resourceConnection = $resourceConnection;
    }

    public function getEntitiesToRemove(array $ids = [])
    {
    }

    public function beforeExecute(array $ids = [])
    {
        $this->resourceConnection->getConnection()->query('UPDATE
	catalog_product_index_price AS q1,
	(
	SELECT
		min(cpp.rule_price) as min_price,
		min(cpp.rule_price) as final_price,
		cpp.website_id,
		cpp.customer_group_id,
		cpe.entity_id
	FROM
		catalog_product_entity cpe
	INNER JOIN catalog_product_entity_int cpei ON
		cpe.row_id = cpei.row_id
	INNER JOIN eav_attribute ea ON
		cpei.attribute_id = ea.attribute_id
	INNER JOIN catalog_product_super_link cpsl ON
		cpe.row_id = cpsl.parent_id
	INNER JOIN catalogrule_product_price cpp ON
		cpp.product_id = cpsl.product_id
	INNER JOIN catalog_product_entity cpe2 ON
		cpe2.entity_id = cpp.product_id
	WHERE
		cpe.type_id = \'configurable\'
		AND ea.attribute_code = \'status\'
	GROUP by
		cpe.entity_id ,
		cpp.website_id ,
		cpp.customer_group_id ) as q2 SET
	q1.min_price = q2.min_price,
	q1.final_price = q2.final_price
WHERE
	q1.website_id = q2.website_id
	AND q1.customer_group_id = q2.customer_group_id
	AND q1.entity_id = q2.entity_id');
    }

    /**
     * @param array $ids
     */
    public function postExecute(array $ids = [])
    {
    }

    /**
     * @return string
     */
    protected function getClass(): string
    {
        return __CLASS__;
    }

    /**
     * @param $entities
     * @param $collection
     * @throws NoSuchEntityException
     */
    protected function write($entities, $collection)
    {
        $this->getConnector()->update($entities, $collection);
    }
}

